#!/usr/bin/env bash
# String Search Algorithms Test Case

usage_exit() {
    echo "$0 [-dsc] [-abonh] [-p PATTERN ] [-f DOMAIN ] [bm ds kmp rk]"
    echo "  -d : Debug Level ( ex. -dd : DEBUG=2)"
    echo "  -s : Show Status" 
    echo "  -c : Continue searching, even if a pattern matched"
    echo
    echo "  -a : Algorithm Trace"
    echo "  -b : Binary Pattern Compare"
    echo "  -o : OK Patterns"
    echo "  -n : NG Patterns"
    echo "  -h : Benchmarking; use Hamlet Script"
    echo
    echo " Algorithms:"
    echo "   bm=Boyer-Moore, ds=Damm-Simple, kmp=Kunuth-Morris-Pratt rk=Rabin-Karp"
    echo
    echo "ex: $0 -p \"ababd\" -f abdom.txt kmp"
    exit
}

ss() {
    declare -n patterns=$1
    local dom="$2"
    local pat p
    for pat in "${patterns[@]}" ; do
        for p in ${PROGRAMS[@]} ; do
            printf "[%3s] %s\n" $p "$pat"
            ./$p ${DEBUG} ${STATS} ${CONT} "$pat" "$dom"
            echo "."
        done
        echo
    done
}

cond() {
    echo "DEBUG=${DEBUG}"
    echo "STATS=${STATS}"
    echo "CONT=${CONT}"
    echo "PROGRAMS=(${PROGRAMS[@]})"
    echo "TEST=${TEST}"
    echo "DOM=${DOM}"
    echo "--------------------------"
}

while getopts :p:dscabonhf: OPT ; do
  case $OPT in
    p)  PAT="${OPTARG}" ;;
    f)  DOM="${OPTARG}" ;;
    d)  DEBUG="${DEBUG--}d"  ;;
    s)  STATS="${STATS--}s"  ;;
    c)  CONT="${CONT--}c"  ;;
    a|b|o|n|h)  TEST="${TEST}${OPT}" ;;
    *)  usage_exit ;; 
  esac
done
shift $((OPTIND - 1))
PROGRAMS=()
for a in "$@" ; do
    case $a in
        "bm"|"ds"|"kmp"|"rk") PROGRAMS+=($a) ;;
    esac
done
[  ${#PROGRAMS[@]} -eq 0 ] && PROGRAMS=(bm ds kmp rk)

cwd=${0%/*}
cd ${cwd}
for t in $(egrep -o . <<<${TEST-a}) ; do
    case "$t" in
        a)  echo "= Algorithm Trace ========"
            [ -z "$PAT" ] && PAT=( "AGCTAGT" )
            DEBUG=${DEBUG--d}
            STATS=${STATS--s}
            DOM=${DOM-dnadom.txt}
            cond
            ss PAT "$DOM"
            ;;

        b)  echo "= Binary Pattern Compare ========"
            [ -z "$PAT" ] && PAT=( "1011" "101110" )
            cond
            DOM=${DOM-bdom.txt}
            ss PAT "$DOM"
            ;;

        o)  echo "= OK Patterns =========="
            [ -z "$PAT" ] && PAT=(
                'ab'
                $'z\nA'
                '89'
                '789'
                '日本語'
                'うえ'
            )
            DOM=${DOM-jdom.txt}
            cond
            ss PAT "$DOM"
            ;;

        n)  echo "= NG Patterns =========="
            [ -z "$PAT" ] && PAT=(
                'aa'
                'zA'
                'ああ'
                '9i8' # 「AWK実践入門」掲載のスクリプトでは、32byte目='F'で疑似ヒット
                '788'
            )
            DOM=${DOM-jdom.txt}
            cond
            ss PAT "$DOM"
            ;;

        h)  echo "= Hamlet Benchmarking ========="
            [ -f "hamlet_TXT_FolgerShakespeare.txt" ] || wget https://shakespeare.folger.edu/downloads/txt/hamlet_TXT_FolgerShakespeare.txt
            [ -z "$PAT" ] && PAT=( 'done' 'Hamlet' 'proved most royal; and' 'therefore'
                       'To be or not to be--that is the question:'
                       'nonetheless' )
            DOM="${DOM-hamlet_TXT_FolgerShakespeare.txt}"
            STATS=${STATS--s}
            CONT=${CONT--c}
            cond
            ss PAT "$DOM"
            ;;
    esac
done

